const express = require('express');
const mysql = require('mysql2');
const bcrypt = require('bcryptjs');
const session = require('express-session');
const path = require('path');
require('dotenv').config();

const app = express();

// Middleware
app.use(express.json());
app.use(express.urlencoded({ extended: true }));
app.use(express.static('public'));
app.set('view engine', 'ejs');

// Session configuration
app.use(session({
    secret: 'your-secret-key',
    resave: false,
    saveUninitialized: false,
    cookie: { secure: false } // Set to true if using HTTPS
}));

// MySQL Connection Pool
const pool = mysql.createPool({
    host: process.env.DB_HOST || 'localhost',
    user: process.env.DB_USER || 'root',
    password: process.env.DB_PASSWORD || '',
    database: process.env.DB_NAME || 'crud_db',
    waitForConnections: true,
    connectionLimit: 10,
    queueLimit: 0
});

// Authentication Middleware
const isAuthenticated = (req, res, next) => {
    if (req.session.user) {
        next();
    } else {
        res.redirect('/');
    }
};

// Routes
app.get('/', (req, res) => {
    if (req.session.user) {
        res.redirect('/dashboard');
    } else {
        res.render('login');
    }
});

// Register routes
app.get('/register', (req, res) => {
    if (req.session.user) {
        res.redirect('/dashboard');
    } else {
        res.render('register');
    }
});

app.post('/register', async (req, res) => {
    const { username, password, confirmPassword } = req.body;

    if (password !== confirmPassword) {
        return res.render('register', { error: 'Passwords do not match' });
    }

    try {
        const [existingUsers] = await pool.promise().query(
            'SELECT * FROM users WHERE username = ?',
            [username]
        );

        if (existingUsers.length > 0) {
            return res.render('register', { error: 'Username already exists' });
        }

        const salt = await bcrypt.genSalt(10);
        const hashedPassword = await bcrypt.hash(password, salt);

        await pool.promise().query(
            'INSERT INTO users (username, password) VALUES (?, ?)',
            [username, hashedPassword]
        );

        res.render('register', { success: 'Registration successful! Please login.' });
    } catch (error) {
        console.error(error);
        res.render('register', { error: 'Server error during registration' });
    }
});

// Login route
app.post('/login', async (req, res) => {
    const { username, password } = req.body;
    
    try {
        const [users] = await pool.promise().query(
            'SELECT * FROM users WHERE username = ?',
            [username]
        );

        if (users.length > 0) {
            const user = users[0];
            const validPassword = await bcrypt.compare(password, user.password);
            
            if (validPassword) {
                req.session.user = {
                    id: user.id,
                    username: user.username
                };
                res.redirect('/dashboard');
            } else {
                res.render('login', { error: 'Invalid password' });
            }
        } else {
            res.render('login', { error: 'User not found' });
        }
    } catch (error) {
        console.error(error);
        res.render('login', { error: 'Server error' });
    }
});

// Logout route
app.get('/logout', (req, res) => {
    req.session.destroy();
    res.redirect('/');
});

// Dashboard routes
app.get('/dashboard', isAuthenticated, (req, res) => {
    res.redirect('/dashboard/products');
});

// Categories routes
app.get('/dashboard/categories', isAuthenticated, async (req, res) => {
    try {
        const [categories] = await pool.promise().query('SELECT * FROM categories ORDER BY created_at DESC');
        res.render('categories', { categories, user: req.session.user });
    } catch (error) {
        console.error(error);
        res.render('categories', { categories: [], error: 'Error fetching categories', user: req.session.user });
    }
});

app.post('/dashboard/categories', isAuthenticated, async (req, res) => {
    const { name, description } = req.body;
    try {
        await pool.promise().query(
            'INSERT INTO categories (name, description) VALUES (?, ?)',
            [name, description]
        );
        res.redirect('/dashboard/categories');
    } catch (error) {
        console.error(error);
        const [categories] = await pool.promise().query('SELECT * FROM categories ORDER BY created_at DESC');
        res.render('categories', { 
            categories, 
            error: 'Error adding category',
            user: req.session.user 
        });
    }
});

app.post('/dashboard/categories/:id', isAuthenticated, async (req, res) => {
    const { id } = req.params;
    const { name, description } = req.body;
    try {
        await pool.promise().query(
            'UPDATE categories SET name = ?, description = ? WHERE id = ?',
            [name, description, id]
        );
        res.redirect('/dashboard/categories');
    } catch (error) {
        console.error(error);
        const [categories] = await pool.promise().query('SELECT * FROM categories ORDER BY created_at DESC');
        res.render('categories', { 
            categories, 
            error: 'Error updating category',
            user: req.session.user 
        });
    }
});

app.delete('/dashboard/categories/:id', isAuthenticated, async (req, res) => {
    const { id } = req.params;
    try {
        await pool.promise().query('DELETE FROM categories WHERE id = ?', [id]);
        res.json({ success: true });
    } catch (error) {
        console.error(error);
        res.json({ success: false, error: 'Error deleting category' });
    }
});

// Products routes
app.get('/dashboard/products', isAuthenticated, async (req, res) => {
    try {
        const [products] = await pool.promise().query(`
            SELECT p.*, c.name as category_name 
            FROM products p 
            LEFT JOIN categories c ON p.category_id = c.id 
            ORDER BY p.created_at DESC
        `);
        const [categories] = await pool.promise().query('SELECT * FROM categories');
        res.render('products', { 
            products, 
            categories,
            user: req.session.user 
        });
    } catch (error) {
        console.error(error);
        res.render('products', { 
            products: [], 
            categories: [],
            error: 'Error fetching products',
            user: req.session.user 
        });
    }
});

app.post('/dashboard/products', isAuthenticated, async (req, res) => {
    const { name, description, price, category_id } = req.body;
    try {
        await pool.promise().query(
            'INSERT INTO products (name, description, price, category_id) VALUES (?, ?, ?, ?)',
            [name, description, price, category_id]
        );
        res.redirect('/dashboard/products');
    } catch (error) {
        console.error(error);
        const [products] = await pool.promise().query(`
            SELECT p.*, c.name as category_name 
            FROM products p 
            LEFT JOIN categories c ON p.category_id = c.id
        `);
        const [categories] = await pool.promise().query('SELECT * FROM categories');
        res.render('products', { 
            products,
            categories,
            error: 'Error adding product',
            user: req.session.user 
        });
    }
});

app.post('/dashboard/products/:id', isAuthenticated, async (req, res) => {
    const { id } = req.params;
    const { name, description, price, category_id } = req.body;
    try {
        await pool.promise().query(
            'UPDATE products SET name = ?, description = ?, price = ?, category_id = ? WHERE id = ?',
            [name, description, price, category_id, id]
        );
        res.redirect('/dashboard/products');
    } catch (error) {
        console.error(error);
        const [products] = await pool.promise().query(`
            SELECT p.*, c.name as category_name 
            FROM products p 
            LEFT JOIN categories c ON p.category_id = c.id
        `);
        const [categories] = await pool.promise().query('SELECT * FROM categories');
        res.render('products', { 
            products,
            categories,
            error: 'Error updating product',
            user: req.session.user 
        });
    }
});

app.delete('/dashboard/products/:id', isAuthenticated, async (req, res) => {
    const { id } = req.params;
    try {
        await pool.promise().query('DELETE FROM products WHERE id = ?', [id]);
        res.json({ success: true });
    } catch (error) {
        console.error(error);
        res.json({ success: false, error: 'Error deleting product' });
    }
});

// Start server
const PORT = process.env.PORT || 3000;
app.listen(PORT, () => {
    console.log(`Server is running on port ${PORT}`);
}); 